/*
 * certkey_engine.h
 */

#include "seal.h"

/***** constants *****/
#define CERTKEY_LEN 128
#define CERTKEY_CERT_DIR "./certs"
#define CERTKEY_KEY_DIR "./certs/my_private"

/***** structure declarations *****/
typedef struct certkey_pubkey_entry_s * CERTKEY_pubkey_entry_p_t;
typedef struct certkey_pubkey_entry_s {
	char uid[SEAL_UID_LEN];			// member's uid
	EVP_PKEY* pub_key;				// member's public key
	CERTKEY_pubkey_entry_p_t prev;
	CERTKEY_pubkey_entry_p_t next;
} CERTKEY_pubkey_entry_t;

typedef struct certkey_pubkey_s * CERTKEY_pubkey_p_t;
typedef struct certkey_pubkey_s {
	CERTKEY_pubkey_entry_p_t head;
	CERTKEY_pubkey_entry_p_t tail;
} CERTKEY_pubkey_t;

typedef struct certkey_control_s * CERTKEY_control_p_t;
typedef struct certkey_control_s {
	CERTKEY_pubkey_p_t pubkey;	
	EVP_PKEY* cert_pri_key;
	EVP_PKEY* cert_pub_key;
	pthread_mutex_t lock;
} CERTKEY_control_t;

/***** function prototypes *****/
CERTKEY_control_p_t CERTKEY_init(char const* my_uid, char* my_passwd);

/*** pubkey management ***/
void CERTKEY_pubkey_install(CERTKEY_control_p_t certkey_engine, 
		char* const* uids, int no_member);
int CERTKEY_pubkey_is_exist(CERTKEY_control_p_t certkey_engine, 
		char const* guid);

/*** signature/verification ***/
int CERTKEY_sign(CERTKEY_control_p_t certkey_engine, void const* msg, 
		int msg_len, char* sig);
int CERTKEY_verify(CERTKEY_control_p_t certkey_engine, char const* sender,
		void const* msg, int msg_len, char* sig, int sig_len);

/*** destroy ***/
void CERTKEY_destroy(CERTKEY_control_p_t certkey_engine);

