/*
 * packet_queue.c
 *
 * - holding all types of packets received from the Spread daemon.
 */

#include "seal.h"

/*
 * Init the message queue
 */
PKTQUEUE_control_p_t PKTQUEUE_init() {
	PKTQUEUE_control_p_t pkt_queue;

	pkt_queue = (PKTQUEUE_control_p_t)malloc(sizeof(PKTQUEUE_control_t));

	pkt_queue->head = NULL;
	pkt_queue->tail = NULL;
	pkt_queue->size = 0;
	pthread_cond_init(&(pkt_queue->cond), NULL);
	pthread_mutex_init(&(pkt_queue->lock), NULL);

	return pkt_queue;
}

/*
 * Enqueue a message
 */
void PKTQUEUE_enq(PKTQUEUE_control_p_t pkt_queue, int service_type, 
		char sender[], int no_guid, char guids[][SEAL_GUID_LEN], 
		int packet_type, int endian_mismatch, char recv_buf[],
		int data_len) {
	
	PKTQUEUE_node_p_t new_node_p;
	int i;

	new_node_p = (PKTQUEUE_node_p_t)malloc(sizeof(PKTQUEUE_node_t));
	memset(new_node_p, 0, sizeof(PKTQUEUE_node_t));

	new_node_p->service_type = service_type;
	strncpy(new_node_p->sender, sender, SEAL_GUID_LEN);
	new_node_p->no_guid = no_guid;
	for (i=0; i<no_guid; ++i) {
		strncpy(new_node_p->guids[i], guids[i], SEAL_GUID_LEN);
	}
	new_node_p->packet_type = packet_type;
	new_node_p->endian_mismatch = endian_mismatch;
	memcpy(new_node_p->recv_buf, recv_buf, data_len);
	new_node_p->data_len = data_len;
	new_node_p->next = NULL;
	
	if (pkt_queue->head == NULL) {
		pkt_queue->head = new_node_p;
	} else {
		pkt_queue->tail->next = new_node_p;
	}	
	pkt_queue->tail = new_node_p;
	pkt_queue->size++;

}

/*
 * Dequeue a message
 */
void PKTQUEUE_deq(PKTQUEUE_control_p_t pkt_queue, int* service_type, 
		char sender[], int* no_guid, char guids[][SEAL_GUID_LEN], 
		int* packet_type, int* endian_mismatch, char recv_buf[],
		int* data_len) {

	PKTQUEUE_node_p_t del_node_p;
	int i;

	if (pkt_queue->size <= 0) {
		debug_err("Dequeue from an empty queue\n");
		return;
	}

	del_node_p = pkt_queue->head;
	*service_type = del_node_p->service_type;
	memcpy(sender, del_node_p->sender, SEAL_GUID_LEN);
	*no_guid = del_node_p->no_guid;
	for (i=0; i<del_node_p->no_guid; ++i) {
		memcpy(guids[i], del_node_p->guids[i], SEAL_GUID_LEN);
	}
	*packet_type = del_node_p->packet_type;
	*endian_mismatch = del_node_p->endian_mismatch;
	memcpy(recv_buf, del_node_p->recv_buf, SEAL_MAX_PKT_SIZE);
	*data_len = del_node_p->data_len;

	if (pkt_queue->head == pkt_queue->tail) {
		pkt_queue->head = NULL;
		pkt_queue->tail = NULL;
		pkt_queue->size = 0;
	} else {
		pkt_queue->head = del_node_p->next;
		pkt_queue->size--;
	}

	free(del_node_p);
}


/*
 * return the size of the queue
 */
int PKTQUEUE_get_size(PKTQUEUE_control_p_t pkt_queue) {
	return pkt_queue->size;
}


/*
 * destroy the message queue
 */
void PKTQUEUE_destroy(PKTQUEUE_control_p_t pkt_queue) {
	PKTQUEUE_node_p_t del_node_p;

	while (pkt_queue->head != NULL) {
		del_node_p = pkt_queue->head;
		pkt_queue->head = del_node_p->next;
		free(del_node_p);
	}	
	free(pkt_queue);
}
