/*
 * packet_queue.h
 */

#include "seal.h"

/* define the message queue node */
typedef struct pktqueue_node_s * PKTQUEUE_node_p_t;
typedef struct pktqueue_node_s {
	int service_type;					// service type (REG / MEMBERSHIP)
	char sender[SEAL_GUID_LEN];			// sender name  
	int no_guid;						// number of guids
	char guids[SEAL_MAX_GUID][SEAL_GUID_LEN];	// array of guids
	int packet_type;					// packet type (in packet.h)
	int endian_mismatch;				// check if endian matches
	char recv_buf[SEAL_MAX_PKT_SIZE];	// received data
	int data_len;                       // data len return (-ve if errors)
	PKTQUEUE_node_p_t next;
} PKTQUEUE_node_t;

/* define the message queue control structure */
typedef struct pktqueue_control_s * PKTQUEUE_control_p_t;
typedef struct pktqueue_control_s {
	PKTQUEUE_node_p_t head;
	PKTQUEUE_node_p_t tail;
	int size;
	pthread_mutex_t lock;	
	pthread_cond_t cond;	
} PKTQUEUE_control_t;

/* function prototypes */
PKTQUEUE_control_p_t PKTQUEUE_init();
void PKTQUEUE_enq(PKTQUEUE_control_p_t msg_queue, int service_type, 
		char sender[], int no_guid, char guids[][SEAL_GUID_LEN], 
		int packet_type, int endian_mismatch, char recv_buf[], int data_len);
void PKTQUEUE_deq(PKTQUEUE_control_p_t msg_queue, int* service_type, 
		char sender[], int* no_guid, char guids[][SEAL_GUID_LEN], 
		int* packet_type, int* endian_mismatch, char recv_buf[], 
		int* data_len);
int PKTQUEUE_get_size(PKTQUEUE_control_p_t msg_queue);
void PKTQUEUE_destroy(PKTQUEUE_control_p_t msg_queue);

